##########################################################
## Title: NASS database update utilizing USDA API       ##
## Author: Shea Winkler                                 ##
## Date Created: 1/3/2019    Date Modified:             ##
## Description: For each county in nebraska, the script ##
## queries the USDA landuse database and appends the    ##
##  ##

import requests
import io
import pandas as pd
import numpy as np
from openpyxl.utils.dataframe import dataframe_to_rows
# xlrd module may be faster
import openpyxl


### Counties List to iterate through ###
counties = [ 'Adams', 'Antelope', 'Arthur', 'Banner', 'Blaine', 'Boone','Box%20Butte',
            'Boyd','Brown','Buffalo','Burt','Butler','Cass','Cedar','Chase','Cherry',
            'Cheyenne','Clay','Colfax','Cuming','Custer','Dakota','Dawes','Dawson',
            'Deuel','Dixon','Dodge','Douglas','Dundy','Fillmore','Franklin',
            'Frontier','Furnas','Gage','Garden','Garfield','Gosper','Grant',
            'Greeley','Hall','Hamilton','Harlan','Hayes','Hitchcock','Holt',
            'Hooker','Howard','Jefferson','Johnson','Kearney','Keith','Keya%20Paha',
            'Kimball','Knox','Lancaster','Lincoln','Logan','Loup','Madison',
            'McPherson','Merrick','Morrill','Nance','Nemaha','Nuckolls',
            'Otoe','Pawnee','Perkins','Phelps','Pierce','Platte','Polk',
            'Red Willow','Richardson','Rock','Saline','Sarpy','Saunders',
            'Scotts%20Bluff','Seward','Sheridan','Sherman','Sioux', 'Stanton',
            'Thayer','Thomas','Thurston','Valley','Washington','Wayne',
            'Webster','Wheeler','York']

## RRCA counties
counties_rrca = ['Adams','Buffalo','Chase','Clay','Dawson','Deuel','Dundy','Franklin',
                 'Frontier','Furnas','Gosper','Harlan','Hayes','Hitchcock','Kearney',
                 'Keith','Lincoln','Nuckolls','Perkins','Phelps','Red Willow','Webster']
i = 0
for county in counties_rrca:
    currYear = 2020
    ### API interaction using requests module ###
    pullCountyData = requests.get("https://quickstats.nass.usda.gov/api/api_GET/?key=CE0B13A8-C7BB-32B6-BA65-E2858B1EF8F9&source_desc=SURVEY&sector_desc=CROPS&group_desc=FIELD%20CROPS&statisticcat_desc=AREA%20HARVESTED&year="+str(currYear)+"&state_alpha=NE&county_name="+county.upper()+"&format=CSV")
    data = pullCountyData.content
    rawData = pd.read_csv(io.StringIO(data.decode('utf-8')))
    if rawData.empty:
        print("No new data for "+county+"in "+str(currYear))
        continue
    ### Logic for Field Crop column ###
    shortName = rawData.loc[:,['short_desc']]
    short = shortName.short_desc.str.replace(' - ACRES HARVESTED','')
    short = short.str.replace(', IRRIGATED','')
    short = short.str.replace(', NON-IRRIGATED','')

    ### Declare new dataframe for new data ###
    df1 = pd.DataFrame()

    ### Add county, year, field crop columns ###
    df1['County'] = rawData['county_name']
    df1['Year'] = rawData['year']
    df1['Field Crop'] = short

    ### Add Irrigated/dry column ###
    listShort = shortName.short_desc.tolist()
    newShort = []
    for x in listShort:
        if ', NON-IRRIGATED' in x:
            newShort.append(x.replace(x,'Dry'))
        elif ', IRRIGATED' in x:
            newShort.append(x.replace(x,'Irrigated'))
        elif 'SUGARBEETS' in x:
            newShort.append(x.replace(x,'Irrigated'))
        else:
            print("NOTE: Line "+x+" is not marked Irrigated/Dry for "+county.replace('%20',' '))
            newShort.append('')

    df1['Irrigated/Dry'] = pd.Series(newShort)

    ### Add Sub-total/duplicate column ###
    subtotal = []
    for x in newShort:
        if x:
            subtotal.append('Sub-Total')
        else:
            subtotal.append('')

    df1['Sub-Total/Duplicate'] = pd.Series(subtotal)

    ### Add Crop Total column ###
    cropTot = []
    for x in listShort:
        if ', NON-IRRIGATED' in x:
            cropTot.append('')

        elif ', IRRIGATED' in x:
            cropTot.append('')
        else:
            cropTot.append('Crop Total')

    df1['Crop Total'] = pd.Series(cropTot)

    ### Add Acres and Description columns ###
    try:
        df1['Acres'] = rawData['Value'].str.replace(',','').astype(np.float)
    except:
        df1['Acres'] = rawData['Value'].astype(np.float)


    df1['Description'] = rawData['short_desc']

    df1 = df1.sort_values('Year',ascending=False)
    df1.to_csv(county+str(currYear)+'.csv')
    ### Read-in current/historical NASS data ###
#     county = county.replace('%20',' ')
#     oldData = pd.read_excel("NASSallCounties.xlsx",sheet_name=county)
#
#     ### Append old data to new data, keeping newest data at the top ###
#     df1 = df1.append(oldData, ignore_index=True,sort=False)
#
#     ### Clear sheet values in order to replace with df1 using openpyxl ###
#     wb = openpyxl.load_workbook(filename="NASSallCounties.xlsx")
#
#     countyWS = wb[county]
#     index = wb.worksheets.index(countyWS)
#     wb.remove(countyWS)
#
#     ### Insert updated values ###
#     newCountyWS = wb.create_sheet(title=county,index=index)
#     ## Might be able to remove openpyxl.utils.dataframe. ##
#     for r in dataframe_to_rows(df1, index=False, header=True):
#         newCountyWS.append(r)
#
#     wb.save('NASSallCounties.xlsx')
#     i += 1
#     print(str(i)+'/'+str(len(rrca_counties)))
# print('The above fraction represents the number of counties for which there is new data for the given year')
